"""Glowing Pancake"""
from .glopan import (
    combine_pdfs,
    config,
    delete_files,
    docx_to_pdf,
    many_ps_to_pdf,
    pdf_convert,
    pdf_to_emf,
    pdf_to_png,
    pdf_to_svg,
    ps_to_pdf,
    split_pdf,
)

# Version
__version__ = '0.0.3'

# Overview over public API
__all__ = [
    'combine_pdfs',
    'config',
    'delete_files',
    'docx_to_pdf',
    'many_ps_to_pdf',
    'pdf_convert',
    'pdf_to_emf',
    'pdf_to_png',
    'pdf_to_svg',
    'ps_to_pdf',
    'split_pdf',
]



from pathlib import Path
import subprocess

import pytest

import glopan

from . import create_files

CURRENT_DRIVE = Path(Path.home().drive)


@pytest.fixture(scope='session')
def ps2pdf_path():
    this_path = (
        Path(CURRENT_DRIVE)
        / '/'
        / 'Program Files'
        / 'gs'
        / 'gs9.26'
        / 'lib'
        / 'ps2pdf.bat'
    )
    return this_path


@pytest.fixture(scope='session')
def pdf2ps_path():
    this_path = (
        Path(CURRENT_DRIVE)
        / '/'
        / 'Program Files'
        / 'gs'
        / 'gs9.26'
        / 'lib'
        / 'pdf2ps.bat'
    )
    return this_path


@pytest.fixture(scope='session')
def inkscape_path():
    this_path = (
        Path(CURRENT_DRIVE)
        / '/'
        / 'Program Files'
        / 'inkscape'
        / 'bin'
        / 'inkscape.exe'
    )
    return this_path


@pytest.fixture(scope='session')
def file_dir(tmpdir_factory):
    return tmpdir_factory.mktemp('files')


@pytest.fixture(scope='session')
def docx_file(file_dir, title: str = 'document_to_convert.docx'):
    file_name = str(file_dir.join(title))
    create_files.create_test_docx(file_name)
    return file_name


@pytest.fixture(scope='session')
def pdf_file_single(file_dir, title: str = 'document_to_convert_single.pdf'):
    file_name = str(file_dir.join(title))
    create_files.create_test_pdf_single(file_name)
    return file_name


@pytest.fixture(scope='session')
def pdf_file_multi(file_dir, title: str = 'document_to_convert_multi.pdf'):
    file_name = str(file_dir.join(title))
    create_files.create_test_pdf_multi(file_name)
    return file_name


@pytest.fixture(scope='session')
def ps_file_single(
    file_dir, pdf2ps_path, title: str = 'ps_document_to_convert_single.ps'
):
    pdf_title = title.replace('.ps', '.pdf')
    pdf_file_name = str(file_dir.join(pdf_title))
    ps_file_name = str(file_dir.join(title))
    create_files.create_test_pdf_single(pdf_file_name)
    arguments = [
        pdf2ps_path,
        pdf_file_name,
        ps_file_name,
    ]
    subprocess.run(arguments, check=False)
    Path(pdf_file_name).unlink()
    return ps_file_name


@pytest.fixture(scope='session')
def ps_file_multi(
    file_dir, pdf2ps_path, title: str = 'ps_document_to_convert_multi.ps'
):
    pdf_title = title.replace('.ps', '.pdf')
    pdf_file_name = str(file_dir.join(pdf_title))
    ps_file_name = str(file_dir.join(title))
    create_files.create_test_pdf_multi(pdf_file_name)
    arguments = [
        pdf2ps_path,
        pdf_file_name,
        ps_file_name,
    ]
    subprocess.run(arguments, check=False)
    Path(pdf_file_name).unlink()
    return ps_file_name


@pytest.fixture(autouse=True)
def set_config(inkscape_path, ps2pdf_path):
    glopan.config.set('inkscape_path', str(inkscape_path))
    glopan.config.set('ps2pdf_path', str(ps2pdf_path))



import json

import glopan
from glopan.config import HOMEPATH, CONFIG_FILENAME, Config


def test_init_with_config_file():
    assert isinstance(glopan.config.config, dict)


def test_init_without_config_file():
    # Remove existing config_file and init new Config
    config_file = HOMEPATH / CONFIG_FILENAME
    temp_config = {**glopan.config.config}
    config_file.unlink()
    test_config = Config()
    assert test_config.config == {
        'inkscape_path': None,
        'ps2pdf_path': None,
    }
    # Reset config_file
    with open(config_file, 'w', encoding='utf-8') as outfile:
        json.dump(temp_config, outfile)
    for key, value in temp_config.items():
        glopan.config.set(key, value)


def test_reset_config():
    config_file = HOMEPATH / CONFIG_FILENAME
    with open(config_file, 'r', encoding='utf-8') as infile:
        temp_config = json.load(infile)
    glopan.config.reset()
    assert glopan.config.config == {
        'inkscape_path': None,
        'ps2pdf_path': None,
    }
    # Reset config_file
    with open(config_file, 'w', encoding='utf-8') as outfile:
        json.dump(temp_config, outfile)
    for key, value in temp_config.items():
        glopan.config.set(key, value)


def test_set_config(inkscape_path, ps2pdf_path):
    glopan.config.set('inkscape_path', str(inkscape_path))
    assert glopan.config.config['inkscape_path'] == str(inkscape_path)
    glopan.config.set('ps2pdf_path', str(ps2pdf_path))
    assert glopan.config.config['ps2pdf_path'] == str(ps2pdf_path)


def test_check_config():
    temp_config = {**glopan.config.config}
    assert glopan.config.check_config('inkscape_path') is True
    assert glopan.config.check_config('asdf') is False
    glopan.config.reset()
    assert glopan.config.check_config('inkscape_path') is False
    glopan.config.set('inkscape_path', 'asdf')
    assert glopan.config.check_config('inkscape_path') is False
    for key, value in temp_config.items():
        glopan.config.set(key, value)



"""Command line interface for Glowing Pancake"""
from pathlib import Path
import typing as t

import rich
import typer

import glopan

VERSION_STR = glopan.__version__
CONTEXT_SETTINGS = {'help_option_names': ['-h', '--help']}

# Initialize Typers
# Main
main = typer.Typer(
    add_completion=False,
    context_settings=CONTEXT_SETTINGS,
    no_args_is_help=True,
    help=f'✨🥞✨ v{VERSION_STR}.',
)

config_typer = typer.Typer(
    name='config',
    context_settings=CONTEXT_SETTINGS,
    no_args_is_help=True,
    help='Set, list or reset the configuration',
)

main.add_typer(config_typer)


@main.command(
    no_args_is_help=True,
)
def convert(
    filename: t.Optional[str] = typer.Argument(
        None, help='The file to convert.'
    ),
    from_format: t.Optional[str] = typer.Option(
        None, '--fromformat', '-ff', help='The format to convert from'
    ),
    to_format: str = typer.Option(
        ..., '--toformat', '-tf', help='The format to convert to'
    ),
):
    """Convert a file to a given format. Provide either the name of a file to
    convert or the format to convert from. If only the format to convert from
    is given, all the files of the given format will be converted.
    """
    if from_format is None and filename is not None:
        from_format = filename[filename.index('.') + 1 :]
    elif from_format is None and filename is None:
        typer.echo(
            'Please provide the name of the file to convert and/or a format to'
            ' convert from.'
        )
        raise typer.Exit()
    if isinstance(from_format, str):
        converter = from_format.lower() + '_to_' + to_format.lower()
    converter_function = getattr(glopan, converter, None)
    if converter_function is not None and isinstance(from_format, str):
        if filename is None:
            filenames = [
                str(this_filename)
                for this_filename in Path().cwd().iterdir()
                if str(this_filename).lower().endswith(from_format)
            ]
            for this_filename in filenames:
                converter_function(this_filename)
        else:
            if Path(filename).exists():
                converter_function(filename)
            else:
                typer.echo(f'The file {filename} does not exist.')
                raise typer.Exit()
    else:
        typer.echo(
            'No available function to convert from '
            f'{from_format} to {to_format}.'
        )
        raise typer.Exit()


@main.command(
    no_args_is_help=True,
)
def combine(
    filenames: t.List[str] = typer.Argument(
        None, help='The files to combine.'
    ),
    file_format: t.Optional[str] = typer.Option(
        None, '--format', '-f', help='Combine all files of this format'
    ),
    outfile: t.Optional[str] = typer.Option(
        None, '--outfile', '-o', help='The name of the combined file'
    ),
):
    """Combine several files to one. Provide either a list of the files to
    combine, or the format of the files to combine.
    """
    if len(filenames) > 0:
        files_to_combine = filenames
        first_file = filenames[0]
        file_format = first_file[first_file.index('.') :].replace('.', '')
    elif len(filenames) == 0 and file_format is not None:
        file_format = file_format.replace('.', '')
        files_to_combine = [
            str(this_file)
            for this_file in Path.cwd().iterdir()
            if str(this_file).lower().endswith(file_format)
        ]
    else:
        typer.echo('Provide at least the format of the files to combine.')
        raise typer.Exit()
    combiner_function = None
    if file_format is not None:
        combiner = 'combine_' + file_format + 's'
        combiner_function = getattr(glopan, combiner, None)

    if outfile is None and file_format is not None:
        outfile = str(
            Path(files_to_combine[0]).parent / ('compilation.' + file_format)
        )

    if combiner_function is not None:
        if files_to_combine and outfile is not None:
            combiner_function(files_to_combine, outfile)
    else:
        typer.echo(f'No available function to combine {file_format} files')
        raise typer.Exit()


# Config commands
@config_typer.command(
    'set',
    no_args_is_help=True,
)
def set_config(
    key: str = typer.Argument(..., help='The key to set'),
    value: str = typer.Argument(..., help='The value of the key to set'),
):
    """Set the value of a configuration key."""
    glopan.config.set(key, value)


@config_typer.command()
def reset():
    """Reset the configuration to an empty state."""
    glopan.config.reset()


@config_typer.command('list')
def list_config():
    """List the current configuration."""
    rich.print(glopan.config.config)


# Create Click object
typer_click_object = typer.main.get_command(main)



"""Collection of functions for Glowing Pancake"""
from pathlib import Path
import subprocess
import typing as t

import docx2pdf
import PyPDF3 as pypdf
import typer

from .config import current_config as config


def combine_pdfs(pdffiles: t.List[str], outfile: str):
    """Combine several PDF files to one.

    Args:
        pdffiles (list): The names of the PDF files to combine.
        outfile (str): The name of the PDF file to write.
    """
    pdf_merger = pypdf.PdfFileMerger()

    for this_pdf in pdffiles:
        pdf_merger.append(this_pdf)

    pdf_merger.write(outfile)
    pdf_merger.close()


def delete_files(files: t.List[str]):
    """Delete a list of files.

    Args:
        files (list): A list of files to delete.
    """
    for file in files:
        Path(file).unlink()


def docx_to_pdf(docxfile: str):
    """Convert a DOCX file to PDF.

    Args:
        docxfile (str): The name of the DOCX file to convert.
    """
    docx2pdf.convert(docxfile)


def many_ps_to_pdf(psfiles: t.List[str]):
    """Convert several Postscript files to PDF using glopan.ps_to_pdf.

    Args:
        psfiles (list): The Postscript files to convert.
    """
    for psfile in psfiles:
        ps_to_pdf(psfile)


def pdf_convert(pdffile: str, outformat: str, outdpi=600):
    """Convert a PDF page to a given format using Inkscape.

    Args:
        pdffile (str): The name of the PDF file to convert.
        outformat (str): The format to convert to.

    Kwargs:
        outdpi (int): The resolution of the outfile, if relevant, in DPI.
    """
    if not config.check_config('inkscape_path'):
        typer.echo('Please set the path to inkscape.exe in the configuration')
        raise typer.Exit()

    arguments = []
    arguments.append(config.config['inkscape_path'])
    arguments.append(f'--export-type={outformat}')
    arguments.append('--pdf-poppler')
    if outformat.lower() == 'png':
        arguments.append(f'--export-dpi={outdpi}')
    arguments.append(pdffile)
    subprocess.run(arguments, check=False)


def pdf_to_emf(pdffile: str):
    """Convert a PDF file to EMF using glopan.pdf_convert."""
    pdf_convert(pdffile, outformat='emf')


def pdf_to_png(pdffile: str, outdpi=600):
    """Convert a PDF file to PNG using glopan.pdf_convert."""
    pdf_convert(pdffile, outformat='png', outdpi=outdpi)


def pdf_to_svg(pdffile: str):
    """Convert a PDF file to SVG using glopan.pdf_convert."""
    pdf_convert(pdffile, outformat='svg')


def ps_to_pdf(psfile: str):
    """Convert a Postscript file to PDF.

    Args:
        psfile (str): The name of the Postscript file.
    """
    if not config.check_config('ps2pdf_path'):
        typer.echo('Please set the path to ps2pdf.bat in the configuration')
        raise typer.Exit()
    filename = psfile[: psfile.index('.')]
    outfile = filename + '.pdf'
    arguments = [config.config['ps2pdf_path'], psfile, outfile]
    subprocess.run(arguments, check=False)


def split_pdf(pdffile: str):
    """Split a PDF file in one file per page.

    Args:
        pdffile (str): The name of the PDF file to split.
    """
    with open(pdffile, 'rb') as pdffile_handle:
        pdf_in = pypdf.PdfFileReader(pdffile_handle)
        num_pages = pdf_in.numPages
        pages = []
        if 'pdf' in pdffile.lower():
            file_first_name = pdffile.lower()[: pdffile.lower().index('.pdf')]

        for page in range(num_pages):
            pages.append(file_first_name + f'_p_{page}' + '.pdf')
            pdf_out = pypdf.PdfFileWriter()
            pdf_out.addPage(pdf_in.getPage(page))

            with open(pages[-1], 'wb') as stream:
                pdf_out.write(stream)

    return pages



from pathlib import Path

import glopan


def test_split_pdf(pdf_file_multi):
    new_filenames = glopan.split_pdf(pdf_file_multi)
    new_files = [Path(new_filename) for new_filename in new_filenames]
    new_files_exist = [new_file.is_file() for new_file in new_files]
    assert all(new_files_exist)


def test_combine_pdf(pdf_file_multi):
    new_filenames = glopan.split_pdf(pdf_file_multi)
    outfile = 'test_combine.pdf'
    glopan.combine_pdfs(new_filenames, outfile)
    new_file = Path(outfile)
    assert new_file.is_file()
    new_file.unlink()


def test_delete_files(pdf_file_multi):
    new_filenames = glopan.split_pdf(pdf_file_multi)
    glopan.delete_files(new_filenames)
    new_files_are_deleted = [
        Path(new_filename).is_file() is not True
        for new_filename in new_filenames
    ]
    assert all(new_files_are_deleted)



import docx

from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import SimpleDocTemplate, PageBreak, Paragraph

STYLES = getSampleStyleSheet()


def create_test_docx(filename: str = 'document_to_convert.docx'):
    """Create dummy docx."""
    docx_document = docx.Document()
    docx_document.add_heading('The title of the document')
    docx_document.add_paragraph('Some interesting text.')
    docx_document.save(filename)


def create_test_pdf_single(filename: str = 'document_to_convert_single.pdf'):
    """Create dummy single page pdf."""
    pdf_document_single = SimpleDocTemplate(filename)
    story = [
        Paragraph(
            'The title of the document',
            STYLES['Heading1']
        ),
        Paragraph(
            'Some interesting text.',
            STYLES['Normal']
        )
    ]
    pdf_document_single.build(story)


def create_test_pdf_multi(filename: str = 'document_to_convert_multi.pdf'):
    pdf_document_multi = SimpleDocTemplate(filename)
    story = [
        Paragraph(
            'The title of the document',
            STYLES['Heading1']
        ),
        Paragraph(
            'A subtitle',
            STYLES['Heading2']
        ),
        Paragraph(
            'Some interesting text on the first page.',
            STYLES['Normal']
        ),
        PageBreak(),
        Paragraph(
            'Another subtitle',
            STYLES['Heading2']
        ),
        Paragraph(
            'Some interesting text on the second page.',
            STYLES['Normal']
        ),
        PageBreak(),
        Paragraph(
            'Another subtitle',
            STYLES['Heading2']
        ),
        Paragraph(
            'Some text on the third page.',
            STYLES['Normal']
        ),
    ]
    pdf_document_multi.build(story)



import io
import os
import contextlib
from pathlib import Path

import typer

import glopan
import glopan.cli as cli


def test_convert(pdf_file_single):
    new_file = pdf_file_single.replace('.pdf', '.emf')
    cli.convert(pdf_file_single, from_format=None, to_format='emf')
    assert Path(new_file).exists()
    Path(new_file).unlink()
    cli.convert(pdf_file_single, from_format='pdf', to_format='emf')
    assert Path(new_file).exists()
    Path(new_file).unlink()


def test_convert_no_filename():
    try:
        cli.convert(None, None, 'emf')
    except typer.Exit:
        assert True
    else:
        assert False


def test_convert_invalid_formats():
    try:
        cli.convert(None, 'invalid_format_1', 'invalid_format_2')
    except typer.Exit:
        assert True
    else:
        assert False


def test_convert_invalid_filename():
    try:
        cli.convert('invalid_filename.pdf', 'pdf', 'emf')
    except typer.Exit:
        assert True
    else:
        assert False


def test_convert_several(file_dir, ps_file_single, ps_file_multi):
    cwd = Path().cwd()
    os.chdir(file_dir)
    relevant_files = [
        this_file
        for this_file in Path().iterdir()
        if str(this_file).lower().endswith('.ps')
    ]
    cli.convert(None, 'ps', 'pdf')
    new_files = [
        Path(str(new_file).lower().replace('.ps', '.pdf'))
        for new_file in relevant_files
    ]
    new_files_exist = [
        new_file.exists() for new_file in new_files
    ]
    assert all(new_files_exist)
    for new_file in new_files:
        new_file.unlink()
    os.chdir(cwd)


def test_combine(pdf_file_single, pdf_file_multi):
    cli.combine([str(pdf_file_single), str(pdf_file_multi)], 'pdf', None)
    new_file = Path(pdf_file_single).parent / 'compilation.pdf'
    assert new_file.exists()
    new_file.unlink()


def test_combine_no_filenames(file_dir, pdf_file_single, pdf_file_multi):
    cwd = Path().cwd()
    os.chdir(file_dir)
    cli.combine([], 'pdf', None)
    new_file = Path(file_dir) / 'compilation.pdf'
    assert new_file.exists()
    new_file.unlink()
    os.chdir(cwd)


def test_combine_no_filename_or_format():
    try:
        cli.combine([], None, None)
    except typer.Exit:
        assert True
    else:
        assert False


def test_combine_no_combiner(ps_file_single, ps_file_multi):
    try:
        cli.combine([str(ps_file_single), str(ps_file_multi)], 'ps', None)
    except typer.Exit:
        assert True
    else:
        assert False


def test_set_config(inkscape_path):
    cli.set_config('inkscape_path', 'asdf')
    assert glopan.config.config['inkscape_path'] == 'asdf'
    cli.set_config('inkscape_path', str(inkscape_path))


def test_reset_config(inkscape_path, ps2pdf_path):
    cli.reset()
    assert glopan.config.config['inkscape_path'] is None
    assert glopan.config.config['ps2pdf_path'] is None
    cli.set_config('inkscape_path', str(inkscape_path))
    cli.set_config('ps2pdf_path', str(ps2pdf_path))


def test_list_config(inkscape_path, ps2pdf_path):
    with io.StringIO() as buffer, contextlib.redirect_stdout(buffer):
        cli.list_config()
        printed_from_cli = buffer.getvalue().replace('\n', '').replace(' ', '')

    with io.StringIO() as buffer, contextlib.redirect_stdout(buffer):
        path_dict = {
            'inkscape_path': str(inkscape_path),
            'ps2pdf_path': str(ps2pdf_path),
        }
        print(path_dict)
        printed_from_test = buffer.getvalue().replace('\n', '').replace(
            ' ', ''
        )
    assert printed_from_cli == printed_from_test



import dotenv
import os
import pathlib
import requests
import subprocess

# Load environment variables from .env
dotenv.load_dotenv()

# Get codecov uploader
url = os.environ.get('codecov_uploader_url')
filename = 'codecov.exe'
request = requests.get(url, allow_redirects=True)
with open(filename, 'wb') as outfile:
    outfile.write(request.content)

# Run uploader
arguments = [
    filename,
    '-t',
    os.environ.get('codecov_token')
]
subprocess.run(arguments)

# Remove uploader
pathlib.Path(filename).unlink(missing_ok=True)



from pathlib import Path

import typer

import glopan


def test_docx_to_pdf(docx_file):
    glopan.docx_to_pdf(docx_file)
    new_file = Path(docx_file.replace('.docx', '.pdf'))
    assert new_file.is_file()
    new_file.unlink()


def test_pdf_to_emf(pdf_file_single):
    glopan.pdf_to_emf(pdf_file_single)
    new_file = Path(pdf_file_single.replace('.pdf', '.emf'))
    assert new_file.is_file()
    new_file.unlink()


def test_pdf_to_png(pdf_file_single):
    glopan.pdf_to_png(pdf_file_single)
    new_file = Path(pdf_file_single.replace('.pdf', '.png'))
    assert new_file.is_file()
    new_file.unlink()


def test_pdf_to_svg(pdf_file_single):
    glopan.pdf_to_svg(pdf_file_single)
    new_file = Path(pdf_file_single.replace('.pdf', '.svg'))
    assert new_file.is_file()
    new_file.unlink()


def test_ps_to_pdf_single(ps_file_single):
    glopan.ps_to_pdf(ps_file_single)
    new_file = Path(ps_file_single.replace('.ps', '.pdf'))
    assert new_file.is_file()
    new_file.unlink()


def test_ps_to_pdf_multi(ps_file_multi):
    glopan.ps_to_pdf(ps_file_multi)
    new_file = Path(ps_file_multi.replace('.ps', '.pdf'))
    assert new_file.is_file()
    new_file.unlink()


def test_many_ps_to_pdf(ps_file_single, ps_file_multi):
    glopan.many_ps_to_pdf([ps_file_single, ps_file_multi])
    new_files = [
        this_file.replace('.ps', '.pdf')
        for this_file in [ps_file_single, ps_file_multi]
    ]
    new_files_exist = [
        Path(new_file).exists() for new_file in new_files
    ]
    assert all(new_files_exist)
    for new_file in new_files:
        Path(new_file).unlink()


def test_ps_to_pdf_no_path(ps_file_single):
    temp_config = {**glopan.config.config}
    glopan.config.reset()
    try:
        glopan.ps_to_pdf(ps_file_single)
    except typer.Exit:
        new_file = Path(ps_file_single.replace('.ps', '.pdf'))
        assert Path(new_file).exists() is False
    for key, value in temp_config.items():
        glopan.config.set(key, value)


def test_pdf_to_emf_no_path(pdf_file_single):
    temp_config = {**glopan.config.config}
    glopan.config.reset()
    try:
        glopan.pdf_to_emf(pdf_file_single)
    except typer.Exit:
        new_file = Path(pdf_file_single.replace('.pdf', '.emf'))
        assert Path(new_file).exists() is False
    for key, value in temp_config.items():
        glopan.config.set(key, value)



"""Configuration for Glowing Pancake"""
import json
import os

from pathlib import Path

HOMEPATH = Path.home()
CONFIG_FILENAME = 'glowing-pancake.json'


class Config:
    """Glowing Pancake configuration"""

    def __init__(self):
        config_file = HOMEPATH / CONFIG_FILENAME
        self.config_file = config_file

        if config_file.exists():
            with open(config_file, 'r', encoding='utf-8') as infile:
                config = json.load(infile)
        else:
            config = {
                'inkscape_path': None,
                'ps2pdf_path': None,
            }
            with open(config_file, 'w', encoding='utf-8') as outfile:
                json.dump(config, outfile)

        self.config = config

        ps2pdf_path = config.get('ps2pdf_path', None)
        if ps2pdf_path is not None:
            os.environ.setdefault(
                'system',
                str(Path(ps2pdf_path).parent.parent / 'bin'),
            )

    def set(self, key: str, value: str):
        """Set a key-value pair in the configuration."""
        if value is not None:
            self.config[key] = value.replace('\\\\', '\\')
        with open(self.config_file, 'w', encoding='utf-8') as config_file:
            json.dump(self.config, config_file)

    def reset(self):
        """Reset configuration"""
        for key in self.config:
            self.config[key] = None
        with open(self.config_file, 'w', encoding='utf-8') as config_file:
            json.dump(self.config, config_file)

    def check_config(self, key):
        """Check if config key is valid"""
        if key in self.config:
            if self.config[key] is None:
                return False
            if not Path(self.config[key]).is_file():
                return False
            return True
        return False


current_config = Config()
